package ru.dbotthepony.mc.otm.client.render

import net.minecraft.client.gui.GuiGraphics
import net.minecraft.world.item.ItemStack

interface IGUIRenderable {
	fun render(guiGraphics: GuiGraphics, x: Float, y: Float, gravity: RenderGravity = RenderGravity.TOP_LEFT)

	/**
	 * Render at specified [x], [y] with fixed [width] x [height]
	 */
	fun render(guiGraphics: GuiGraphics, x: Float, y: Float, width: Float, height: Float)

	fun composeBefore(other: IGUIRenderable): IGUIRenderable {
		return object : IGUIRenderable {
			override fun render(guiGraphics: GuiGraphics, x: Float, y: Float, gravity: RenderGravity) {
				this@IGUIRenderable.render(guiGraphics, x, y, gravity)
				other.render(guiGraphics, x, y, gravity)
			}

			override fun render(guiGraphics: GuiGraphics, x: Float, y: Float, width: Float, height: Float) {
				this@IGUIRenderable.render(guiGraphics, x, y, width, height)
				other.render(guiGraphics, x, y, width, height)
			}
		}
	}

	fun composeAfter(other: IGUIRenderable): IGUIRenderable {
		return other.composeBefore(this)
	}

	companion object {
		fun of(value: ItemStack, width: Float = 16f, height: Float = 16f): IGUIRenderable {
			return ItemStackIcon(value, width, height)
		}

		fun of(value: IGUIRenderable): IGUIRenderable {
			return value
		}
	}
}

data class ItemStackIcon(private val itemStack: ItemStack, val width: Float = 16f, val height: Float = 16f) : IGUIRenderable {
	override fun render(guiGraphics: GuiGraphics, x: Float, y: Float, gravity: RenderGravity) {
		return render(guiGraphics, gravity.x(x, width), gravity.y(y, height), width, height)
	}

	override fun render(guiGraphics: GuiGraphics, x: Float, y: Float, width: Float, height: Float) {
		if (x % 1f == 0f && y % 1f == 0f && width == 16f && height == 16f) {
			guiGraphics.renderFakeItem(itemStack, x.toInt(), y.toInt())
		} else {
			val pose = guiGraphics.pose()

			pose.pushPose()
			pose.translate(x % 1f, y % 1f, 0f)

			if (width != 16f || height != 16f)
				pose.scale(width / 16f, height / 16f, 1f)

			guiGraphics.renderFakeItem(itemStack, x.toInt(), y.toInt())
			pose.popPose()
		}
	}
}
